package dashboard

import (
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/aaa"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/dashboard/model"
)

type summary struct {
	Table
	client client
	create string `name:"create title* space index* query field score" role:"leader"`
	modify string `name:"modify title* space index* query field score" role:"leader"`
	remove string `name:"remove" role:"leader"`
	scan   string `name:"scan" role:"leader"`
}

func (s summary) Create(m *ice.Message, arg ...string) { s.ValueCreate(m, arg...) }
func (s summary) Modify(m *ice.Message, arg ...string) { s.ValueModify(m, arg...) }
func (s summary) Remove(m *ice.Message, arg ...string) { s.ValueRemove(m, arg...) }
func (s summary) Scan(m *ice.Message, arg ...string) {
	client_uid := kit.Select(m.Option(model.CLIENT_UID), arg, 0)
	msg := m.Cmd(s.client, s.Select, model.UID, client_uid)
	switch ClientType(kit.Int(msg.Append(model.TYPE))) {
	case ClientPlace:
		s.List(m, client_uid).Table(func(value ice.Maps) {
			msg := m.Cmd(web.SPACE, value[web.SPACE], value[ctx.INDEX], mdb.SELECT, kit.Simple(kit.UnMarshal(value[model.QUERY])), kit.Dict(
				mdb.SELECT, kit.Select("count(*)", value[mdb.FIELD]),
				ice.MSG_USERROLE, aaa.TECH,
			))
			s.Update(m, kit.Dict(model.VALUE, msg.Append(msg.Append(ice.MSG_APPEND))), model.UID, value[model.UID])
		})
	case ClientMySQL:
		s.Open(m, func(db *gorm.DB) {
			s.List(m, client_uid).Table(func(value ice.Maps) {
				db = db.Table(s.TableName(kit.Join(kit.Slice(kit.Split(value[ctx.INDEX], "."), -2), ".")))
				kit.If(value[model.QUERY], func(p string) { s.Where(m, db, kit.Simple(kit.UnMarshal(p))...) })
				msg := s.Rows(m.Spawn(), db.Select(kit.Select("count(*)", value[mdb.FIELD])))
				s.Update(m, kit.Dict(model.VALUE, msg.Append(msg.Append(ice.MSG_APPEND))), model.UID, value[model.UID])
			})
		})
	}
}
func (s summary) List(m *ice.Message, arg ...string) *ice.Message {
	s.Orders(m, model.SCORE, s.Desc(model.CREATED_AT))
	s.Select(m, model.CLIENT_UID, arg[0]).PushAction(s.Modify, s.Remove).Action(s.Create, s.Scan)
	return s.Button(m, "").Display("").DisplayCSS("")
}

func init() { ice.TeamCtxCmd(summary{}) }

func (s summary) Open(msg *ice.Message, cb func(*gorm.DB)) {
	dsn := kit.Format("%s:%s@tcp(%s:%s)/%s?charset=utf8mb4&parseTime=True", msg.Append(model.USERNAME), msg.Append(model.PASSWORD), msg.Append(model.HOST), msg.Append(model.PORT), msg.Append(model.DATABASE))
	if db, err := gorm.Open(mysql.Open(dsn), &gorm.Config{Logger: logger.Default.LogMode(logger.Info)}); !msg.Warn(err) {
		cb(db)
	}
}
