package production

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/production/model"
)

type plan struct {
	Table
	issue       issue
	order       string `data:"1"`
	fields      string `data:"title,content,plan_status,begin_time,end_time"`
	create      string `name:"create title* content* begin_time*:select@date end_time*:select@date" role:"leader"`
	modify      string `name:"modify title* content* begin_time*:select@date end_time*:select@date" role:"leader"`
	remove      string `name:"remove" role:"leader"`
	process     string `name:"process" role:"leader"`
	finish      string `name:"finish" role:"leader"`
	issueCreate string `name:"issueCreate issue_title* issue_content* issue_type*:select level*:select" role:"leader,worker"`
}

func (s plan) Create(m *ice.Message, arg ...string) { s.ValueCreate(m, arg...) }
func (s plan) Modify(m *ice.Message, arg ...string) { s.ValueModify(m, arg...) }
func (s plan) Remove(m *ice.Message, arg ...string) { s.ValueRemove(m, arg...) }
func (s plan) List(m *ice.Message, arg ...string) {
	s.ValueList(m, arg).Table(func(value ice.Maps) {
		button := []ice.Any{}
		switch PlanStatus(kit.Int(value[model.PLAN_STATUS])) {
		case PlanCreate:
			button = append(button, s.Modify, s.Process)
		case PlanProcess:
			button = append(button, s.IssueCreate, s.Modify, s.Finish)
		default:
			if m.Option(model.MARKET_UID) == "" {
				button = append(button, s.MarketInsert)
			}
		}
		m.PushButton(button...)
	}).Display("")
}
func (s plan) Process(m *ice.Message, arg ...string) { s.changeStatus(m, PlanCreate, PlanProcess) }
func (s plan) Finish(m *ice.Message, arg ...string)  { s.changeStatus(m, PlanProcess, PlanFinish) }
func (s plan) IssueCreate(m *ice.Message, arg ...string) {
	m.Cmd(s.issue, s.Create, arg, model.PLAN_UID, m.Option(model.UID))
}

func init() { ice.TeamCtxCmd(plan{}) }

func (s plan) changeStatus(m *ice.Message, from, to PlanStatus, arg ...string) {
	s.ChangeStatus(m, int(from), int(to))
}

type PlanStatus int

const (
	PlanCreate PlanStatus = iota
	PlanProcess
	PlanFinish
)

var PlanStatusList = map[PlanStatus]string{
	PlanCreate:  "create",
	PlanProcess: "process",
	PlanFinish:  "finish",
}

func (s PlanStatus) String() string { return PlanStatusList[s] }
