package production

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/production/model"
)

type task struct {
	Table
	Case       Case
	order      string `data:"3"`
	fields     string `data:"title,content,begin_time,end_time,task_status,issue_uid,plan_uid,user_uid"`
	create     string `name:"create title* content* begin_time*:select@date end_time*:select@date" role:"leader,worker"`
	modify     string `name:"modify title* content* begin_time*:select@date end_time*:select@date" role:"leader,worker"`
	remove     string `name:"remove" role:"leader,worker"`
	process    string `name:"process" role:"leader,worker"`
	finish     string `name:"finish" role:"leader,worker"`
	planBind   string `name:"planBind plan_uid*" role:"leader"`
	caseCreate string `name:"caseCreate title* content*" role:"leader,worker"`
}

func (s task) Create(m *ice.Message, arg ...string) {
	s.ValueCreate(m, arg...)
	s.SendMessage(m, "", "")
	s.DashboardUpdate(m)
}
func (s task) Remove(m *ice.Message, arg ...string) { s.ValueRemove(m, arg...); s.DashboardUpdate(m) }
func (s task) Modify(m *ice.Message, arg ...string) { s.ValueModify(m, arg...) }
func (s task) List(m *ice.Message, arg ...string) {
	s.ValueList(m, arg).Table(func(value ice.Maps) {
		button := []ice.Any{}
		switch TaskStatus(kit.Int(value[model.TASK_STATUS])) {
		case TaskCreate:
			if value[model.PLAN_UID] == "" {
				button = append(button, s.PlanBind)
			}
			button = append(button, s.Process)
		case TaskProcess:
			button = append(button, s.CaseCreate)
			button = append(button, s.Finish)
		case TaskFinish:
			kit.If(len(arg) == 2, func() { s.DoneMessage(m) })
		}
		m.PushButton(button...)
	}).Display("")
	s.SelectJoinPlan(m)
	s.SelectJoinUser(m)
}
func (s task) Process(m *ice.Message, arg ...string)  { s.changeStatus(m, TaskCreate, TaskProcess) }
func (s task) Finish(m *ice.Message, arg ...string)   { s.changeStatus(m, TaskProcess, TaskFinish) }
func (s task) PlanBind(m *ice.Message, arg ...string) { s.ValueModify(m, arg...) }
func (s task) CaseCreate(m *ice.Message, arg ...string) {
	m.Cmd(s.Case, s.Create, arg, model.TASK_UID, m.Option(model.UID), m.OptionSimple(model.PLAN_UID))
}

func init() { ice.TeamCtxCmd(task{}) }

func (s task) changeStatus(m *ice.Message, from, to TaskStatus, arg ...string) {
	s.ChangeStatus(m, int(from), int(to))
	s.DashboardUpdate(m)
}

type TaskStatus int

const (
	TaskCreate TaskStatus = iota
	TaskProcess
	TaskFinish
)

var TaskStatusList = map[TaskStatus]string{
	TaskCreate:  "create",
	TaskProcess: "process",
	TaskFinish:  "finish",
}

func (s TaskStatus) String() string { return TaskStatusList[s] }
